﻿Type=StaticCode
Version=1.60
@EndOfDesignText@
'Code module
'Subs in this code module will be accessible from all modules.
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.

	#Region Private Globals
	Dim Version As String :  Version = "1.0C"

	' set to True to see full logged information
	Dim LOGGING As Boolean : LOGGING = False
	


	Dim manager As UsbManager	
	Dim devices() As UsbDevice
	Dim connection As UsbDeviceConnection
	Dim outEndpoint, inEndpoint, intEndpoint As UsbEndpoint
	Dim pl2303index As Int
	Dim interface As UsbInterface
	Dim MaxInPacketLength As Int
	
	Dim RxCallbackActivity As String 'Name of Activity that handles the RX callback.
	Dim RxCallbackSub As String 'Name of the RX callback sub.
	
	Dim rawdescs() As Byte
	Dim pl2303vendor As Int : pl2303vendor = 0x67b
	Dim pl2303productid As Int : pl2303productid = 0x2303 
	Dim chiptype As Int ' 0, 1, or 2 for HX
	
	Dim status As Int
	Dim inbuffer(4096) As Byte
	Dim inbufferinptr, inbufferoutptr As Int
	
	Dim TIMEOUT As Int : TIMEOUT = 1 ' 1mS timeout for control transfers
	Dim MAX_TIMER_READS As Int : MAX_TIMER_READS = 1
	Dim TIMER_INTERVAL As Int : TIMER_INTERVAL = 100
	
	Dim SET_LINE_REQUEST_TYPE As Int : SET_LINE_REQUEST_TYPE = 0x21
	Dim SET_LINE_REQUEST As Int : SET_LINE_REQUEST = 0x20
	
	Dim SET_CONTROL_REQUEST_TYPE As Int : SET_CONTROL_REQUEST_TYPE = 0x21
	Dim SET_CONTROL_REQUEST As Int : SET_CONTROL_REQUEST = 0x22
	Dim CONTROL_DTR As Int : CONTROL_DTR = 0x01
	Dim CONTROL_RTS As Int : CONTROL_RTS = 0x02
	
	Dim BREAK_REQUEST_TYPE As Int : BREAK_REQUEST_TYPE = 0x21
	Dim BREAK_REQUEST As Int : BREAK_REQUEST = 0x23
	Dim BREAK_ON As Int : BREAK_ON = 0xffff
	Dim BREAK_OFF As Int : BREAK_OFF = 0x0000
	
	Dim GET_LINE_REQUEST_TYPE As Int : GET_LINE_REQUEST_TYPE = 0xa1
	Dim GET_LINE_REQUEST As Int : GET_LINE_REQUEST =  0x21
	
	Dim VENDOR_WRITE_REQUEST_TYPE As Int : VENDOR_WRITE_REQUEST_TYPE = 0x40
	Dim VENDOR_WRITE_REQUEST As Int : VENDOR_WRITE_REQUEST = 0x01
	
	Dim VENDOR_READ_REQUEST_TYPE As Int : VENDOR_READ_REQUEST_TYPE = 0xc0
	Dim VENDOR_READ_REQUEST As Int : VENDOR_READ_REQUEST = 0x01
	
	#End Region
	
	#Region Public Globals
	
	' Type for the UART setup data
	Type UARTdata (rate As Int, stopbits As Int, parity As Int, databits As Int)
	
	' Type for the received data
	Type RxDataBlock (data() As Byte, bytecount As Int)	
	
	' Data blocks received are placed in this list and the provided Sub in the activity is called
	Dim RxData As List

	' UART setup constants, should be self-explanatory
	Dim UART_ERRORS_MASK As Int : UART_ERRORS_MASK = 0x74 
	Dim UART_STATUS_MASK As Int : UART_STATUS_MASK = 0x8b
	
	Dim UART_DCD As Int : UART_DCD = 0x01
	Dim UART_DSR As Int : UART_DSR = 0x02
	Dim UART_BREAK_ERROR As Int : UART_BREAK_ERROR = 0x04
	Dim UART_RING As Int : UART_RING = 0x08
	Dim UART_FRAME_ERROR As Int : UART_FRAME_ERROR = 0x10
	Dim UART_PARITY_ERROR As Int : UART_PARITY_ERROR = 0x20
	Dim UART_OVERRUN_ERROR As Int : UART_OVERRUN_ERROR = 0x40
	Dim UART_CTS As Int : UART_CTS = 0x80
	
	Dim UART_DTR As Int : UART_DTR = 0x01
	Dim UART_RTS As Int : UART_RTS = 0x02

	Dim UART_DCD As Int : UART_DCD = 0x01
	Dim UART_DSR As Int : UART_DSR = 0x02
	Dim UART_BREAK_ERROR As Int : UART_BREAK_ERROR = 0x04
	Dim UART_RING As Int : UART_RING = 0x08
	Dim UART_FRAME_ERROR As Int : UART_FRAME_ERROR = 0x10
	Dim UART_PARITY_ERROR As Int : UART_PARITY_ERROR = 0x20
	Dim UART_OVERRUN_ERROR As Int : UART_OVERRUN_ERROR = 0x40
	Dim UART_CTS As Int : UART_CTS = 0x80
	
	Dim PARITY_NONE As Int : PARITY_NONE = 0
	Dim PARITY_ODD As Int : PARITY_ODD = 1
	Dim PARITY_EVEN As Int : PARITY_EVEN = 2
	Dim PARITY_MARK As Int : PARITY_MARK = 3
	Dim PARITY_SPACE As Int : PARITY_SPACE = 4
	
	Dim STOPBITS_1 As Int : STOPBITS_1 = 0
	Dim STOPBITS_15 As Int : STOPBITS_15 = 1
	Dim STOPBITS_2 As Int : STOPBITS_2 = 2
	
	Dim DATABITS_5 As Int : DATABITS_5 = 5 
	Dim DATABITS_6 As Int : DATABITS_6 = 6 
	Dim DATABITS_7 As Int : DATABITS_7 = 7
	Dim DATABITS_8 As Int : DATABITS_8 = 8 
	
	Dim RATE_75 As Int : RATE_75 = 75
	Dim RATE_150 As Int : RATE_150 = 150
	Dim RATE_300 As Int : RATE_300 = 300
	Dim RATE_600 As Int : RATE_600 = 600
	Dim RATE_1200 As Int : RATE_1200 = 1200
	Dim RATE_1800 As Int : RATE_1800 = 1800
	Dim RATE_2400 As Int : RATE_2400 = 2400
	Dim RATE_4800 As Int : RATE_4800 = 4800
	Dim RATE_7200 As Int : RATE_7200 = 7200
	Dim RATE_9600 As Int : RATE_9600 = 9600
	Dim RATE_14400 As Int : RATE_14400 = 14400
	Dim RATE_19200 As Int : RATE_19200 = 19200
	Dim RATE_28800 As Int : RATE_28800 = 28800
	Dim RATE_38400 As Int : RATE_38400 = 38400
	Dim RATE_57600 As Int : RATE_57600 = 57600
	Dim RATE_115200 As Int : RATE_115200 = 115200
	Dim RATE_230400 As Int : RATE_230400 = 230400
	Dim RATE_460800 As Int : RATE_460800 = 468000
	Dim RATE_614400 As Int : RATE_614400 = 614400
	Dim RATE_921600 As Int : RATE_921600 = 921600
	Dim RATE_1228800 As Int : RATE_1228800 = 1228800
	
	#End Region
		
End Sub ' Globals

#Region Public Subs

' PUBLIC Check there is a PL2303 present, returns True if so
Sub Find As Boolean
	manager.Initialize
	pl2303index = -1
	devices = manager.GetDevices
	For i = 0 To devices.Length - 1
		LogMsg("Device " & i & " = " & devices(i).VendorId & " " & devices(i).ProductId)
		If devices(i).VendorId = pl2303vendor AND devices(i).ProductId = pl2303productid Then
			pl2303index = i	
			Return True
		End If
	Next
	Return False
End Sub

' PUBLIC Check if the application has permission to access USB, returns True if so
Sub CheckPermission As Boolean
	Return manager.HasPermission(devices(pl2303index))
End Sub

' PUBLIC Request that the application has permission to access USB, returns true if granted
Sub RequestPermission As Boolean
	manager.RequestPermission(devices(pl2303index))
End Sub

' PUBLIC Open the PL2303 for use
' Values of PollInterval and MaxPollsPerInterval to avoid missing characters depend upon the baud rate
' and also the amount (unknown) of buffering in the device and Android USB driver and
' it also depends on the burstiness of the data
' e.g 4800 baud is approximately 480 characters per second
' as the maximum read packet length is 64 we need to poll once about 8 times per second to not miss data
'
' RxModule and RxSub specify the Sub to call when data is received 
Sub Open(PollInterval As Int, MaxPollsPerInterval As Int, RxModule As String, RxSub As String)
	
	RxCallbackActivity = RxModule
	RxCallbackSub = RxSub
	
	MAX_TIMER_READS = MaxPollsPerInterval
	TIMER_INTERVAL = PollInterval

	RxData.Initialize
	
	interface = devices(pl2303index).GetInterface(0)
	connection = manager.OpenDevice(devices(pl2303index), interface, True)

	Dim Obj1 As Reflector
	Obj1.Target = connection
	Obj1.Target = Obj1.GetField("connection")
	rawdescs = Obj1.RunMethod("getRawDescriptors")

	For j = 0 To interface.EndpointCount - 1
		Dim endpoint As UsbEndpoint
		endpoint = interface.GetEndpoint(j)
		If endpoint.Type = manager.USB_ENDPOINT_XFER_BULK Then
			If endpoint.Direction = manager.USB_DIR_IN Then 
				inEndpoint = endpoint
				MaxInPacketLength = inEndpoint.MaxPacketSize
			Else If endpoint.Direction = manager.USB_DIR_OUT Then
				outEndpoint = endpoint
			End If
		Else If endpoint.Type = manager.USB_ENDPOINT_XFER_INT Then
			intEndpoint = endpoint					
		End If
	Next
	
	LogBytes("RawDescriptors (" & rawdescs.Length & ") :", rawdescs, True)
	LogMsg("Int " & intEndpoint)
	LogMsg("In " & inEndpoint)
	LogMsg("Out " & outEndpoint)
	
	' some undocumented magic initialisation as found in Linux driver code
	If (rawdescs(4) = 0x02) Then  ' bDeviceClass		
			chiptype = 0	' type 0	
	Else If (rawdescs(7) = 0x40) Then	' bMaxPacketSize0
			chiptype = 2	' type HX
			' bit of unknown magic Linux code says "reset upstream data pipes"
			VendorWrite(8, 0)
			VendorWrite(9, 0)
	Else If (rawdescs(4) = 0x00) Then ' bDeviceClass		
			chiptype = 1	' type1 	
	Else If (rawdescs(4) = 0xFF) Then ' bDeviceClass		
			chiptype = 1 ' type 1
	End If			
	Dim buf(1) As Byte
	VendorRead(0x8484, 0, buf)	
	VendorWrite(0x0404, 0)	
	VendorRead(0x8484, 0, buf)	
	VendorRead(0x8383, 0, buf)	
	VendorRead(0x8484, 0, buf)	
	VendorWrite(0x0404, 1)	
	VendorRead(0x8484, 0, buf)	
	VendorRead(0x8383, 0, buf)	
	VendorWrite(0, 1)	
	VendorWrite(1, 0)	
	If (chiptype = 2) Then	
			VendorWrite(2, 0x44)	
	Else	
			VendorWrite(2, 0x24)	
	End If	
	' end of undocumented magic
	
	StartService("USBpl2303Service")

End Sub

' PUBLIC Close the connection to the PL2303
Sub Close
	StopService("USBpl2303Service") ' do this first before closing connection
End Sub


' PUBLIC Write data asynchronously.
Sub WriteData(outdata() As Byte, length As Int) As Int
	' synchronous write should return the number of bytes written OR -ve If error.
	'Return connection.BulkTransfer(outEndpoint, outdata, length, timeout)
	' however the return value cannot be relied upon. It can return -1 having actually sent the data!
	' therefore we use asynchronous which seems to work
	
	' asynchronous write
	Dim request As UsbRequest
	request.Initialize(connection, outEndpoint)
	request.Queue(outdata, length)
End Sub

' PUBLIC Set up the characteristics of the the serial port as a UARTdata
' no error checking is performed, that is the responsibility of the calling code
Sub SetLineData(data As UARTdata) 
	Dim buf(7) As Byte
	buf(0) = Bit.And(data.rate, 0xff)
	buf(1) = Bit.And(Bit.ShiftRight(data.rate, 8), 0xff)
	buf(2) = Bit.And(Bit.ShiftRight(data.rate, 16), 0xff)
	buf(3) = Bit.And(Bit.ShiftRight(data.rate, 24), 0xff)
	buf(4) = data.stopbits
	buf(5) = data.parity
	buf(6) = data.databits
	LogBytes("Uart values out : ", buf, True)
	connection.ControlTransfer(SET_LINE_REQUEST_TYPE, SET_LINE_REQUEST, 0, 0, buf, 7, TIMEOUT)
End Sub

' PUBLIC Get the characteristics of the the serial port as a UARTdata
Sub GetLineData As UARTdata
	Dim buf(7) As Byte
	Dim data As UARTdata
	Dim baud As Int
	connection.ControlTransfer(GET_LINE_REQUEST_TYPE, GET_LINE_REQUEST, 0, 0, buf, 7, TIMEOUT)
	LogBytes("Uart values in : ", buf, True)
	For i = 3 To 0 Step -1
		baud = Bit.ShiftLeft(baud, 8)
		baud = Bit.Or(baud, Bit.And(buf(i), 0xff))
	Next
	data.rate = baud
	data.stopbits = buf(4)
	data.parity = buf(5)
	data.databits = buf(6)
	Return data
End Sub

' PUBLIC Get the present status of the UART. At the moment it will return 0 until a status change occurs
' this is because status is only sent on a change and I haven't found a way to prod it to get the inital value
'	7			6				5			4			3			2			1		0
'CTS	OVRRUN	PRTY	FRAME	RING	BREAK	DSR	DCD
'			ERROR		ERROR	ERROR		    ERROR
' The error bits are stucky and should be reset using ResetStatusErrorBits
Sub GetStatus As Int
	Return status
End Sub

' PUBLIC Reset the error bits of the present status. They are purposely made "sticky"
Sub ResetStatusErrorBits
	status = Bit.And(status, UART_STATUS_MASK)
End Sub

' PUBLIC Set the state of DTR and RTS
Sub SetDtrRts(dtr As Boolean, rts As Boolean)
	Dim value As Int
	If dtr Then value = Bit.Or(value, UART_DTR)
	If rts Then value = Bit.Or(value, UART_RTS)
	connection.ControlTransfer(SET_CONTROL_REQUEST_TYPE, SET_CONTROL_REQUEST, value, 0, Null, 0, TIMEOUT)
End Sub

' PUBLIC Set the state of Break
Sub SetBreak(break As Boolean)
	Dim value As Int
	If break Then
		value = BREAK_ON
	Else
		value = BREAK_OFF
	End If
	connection.ControlTransfer(BREAK_REQUEST_TYPE, BREAK_REQUEST, value, 0, Null, 0, TIMEOUT)
End Sub


#Region Private Subs

Sub VendorRead(value As Int, index As Int, bytes() As Byte)
	connection.ControlTransfer(VENDOR_READ_REQUEST_TYPE, VENDOR_READ_REQUEST, value, index, bytes, 1, TIMEOUT)
End Sub

Sub VendorWrite(value As Int, index As Int)
	connection.ControlTransfer(VENDOR_WRITE_REQUEST_TYPE, VENDOR_WRITE_REQUEST, value, index, Null, 0, TIMEOUT)
End Sub

Sub LogMsg(msg As String)
	If LOGGING Then
		Log(msg)
	End If
End Sub

Sub LogBytes(prefix As String, bytes() As Byte, hex As Boolean)
	If LOGGING Then
		Dim conv As ByteConverter
		Dim msg, msghex As String
		Dim byte1(1) As Byte
		msg = prefix
		msghex = prefix
		For i = 0 To bytes.Length -1
			byte1(0) = bytes(i)
			msg = msg & " " & bytes(i)
			msghex = msghex & " " & conv.HexFromBytes(byte1)
		Next
		If hex Then
			Log(msghex)
		Else
			Log(msg)
		End If
	End If
End Sub

#End Region
