﻿Type=Service
Version=1.60
StartAtBoot=False
@EndOfDesignText@
'Service module
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	
	Dim Version As String :  Version = "1.0S"
	
	Dim Timer1 As Timer
	
End Sub

Sub Service_Create
	Timer1.Initialize("Timer1", USBpl2303.TIMER_INTERVAL)
End Sub

Sub Service_Start(StartingIntent As Intent)
	USBpl2303.connection.StartListening("USBpl2303")
	' status doesn't seem to work with Connection.BulkTransfer, returns -1 with array unchanged
	' so we need to use an asynchronous interrupt request to get status changes
	
	' asynchronous read works but there is no way to determine the number of valid bytes so it is useless
	' so reading is done synchronously by a timer
	
	' asynchronous write works fine but the synchronous version is pretty quick so we use that
	
	' kick off the interrupt receiver	and the read timer
	SendIntRequest
	Timer1.Interval = USBpl2303.TIMER_INTERVAL
	Timer1.Enabled = True
End Sub

Sub Service_Destroy
	Timer1.Enabled = False
	USBpl2303.connection.StopListening
End Sub

Sub Timer1_Tick
	Dim num, reads As Int
	Dim msg As String
	Dim mS As Long
	Dim gotdata As Boolean
	reads = 1
	ms = DateTime.Now
	Do While reads <= USBpl2303.MAX_TIMER_READS
		Dim indata(USBpl2303.MaxInPacketLength) As Byte
		num = USBpl2303.connection.BulkTransfer(USBpl2303.inEndpoint, indata, indata.length, USBpl2303.TIMEOUT)
		If num < 1 Then
			Exit ' no data available
		End If
		gotdata = True
		Dim rxdata As RxDataBlock
		rxdata.bytecount = num
		rxdata.data = indata
		USBpl2303.RxData.Add(rxdata)
		msg = "Bytes read(" & reads & ") = " & num & " : "
		LogBytes(msg, indata, True)
		reads = reads + 1
	Loop
	If gotdata Then	
		LogMsg("Reading for  " & (DateTime.Now - mS) & " mS")
		LogMsg("Calling " & USBpl2303.RxCallbackActivity & "." & USBpl2303.RxCallbackSub)
		CallSub(USBpl2303.RxCallbackActivity, USBpl2303.RxCallbackSub)
	End If
End Sub

Sub USBpl2303_NewData (Request As UsbRequest, InDirection As Boolean)	
	Try
		If USBpl2303.Connection.IsInitialized = False Then
			Return 'Might happen after we close the connection
		End If
		If InDirection = False Then 
			USBpl2303.connection.ContinueListening
			Return 'don't handle OUT requests
		End If
		Dim bits As Int
		Dim errors As Int
		errors = Bit.And(USBpl2303.status, USBpl2303.UART_ERRORS_MASK) ' keep any errors		
		bits = Request.Buffer(8)
		USBpl2303.status = Bit.Or(bits, errors)
		LogBytes("Status : ", Request.Buffer, True)
		If USBpl2303. connection.IsInitialized Then
			SendIntRequest
		End If
		USBpl2303.connection.ContinueListening
	Catch
	End Try
End Sub

Sub SendIntRequest
	LogMsg("Sending Int request")
	Dim request As UsbRequest
	request.Initialize(USBpl2303.connection, USBpl2303.intEndpoint)
	Dim Data(USBpl2303.intEndpoint.MaxPacketSize) As Byte
	request.Queue(Data, Data.Length)
End Sub

Sub LogMsg(msg As String)
	If USBpl2303.LOGGING Then
		Log(msg)
	End If
End Sub

Sub LogBytes(prefix As String, bytes() As Byte, hex As Boolean)
	If USBpl2303.LOGGING Then
		Dim conv As ByteConverter
		Dim msg, msghex As String
		Dim byte1(1) As Byte
		msg = prefix
		msghex = prefix
		For i = 0 To bytes.Length -1
			byte1(0) = bytes(i)
			msg = msg & " " & bytes(i)
			msghex = msghex & " " & conv.HexFromBytes(byte1)
		Next
		If hex Then
			Log(msghex)
		Else
			Log(msg)
		End If
	End If
End Sub

